;========================================================================================
; Toolbar Test Bed
;
; Now With Tasty Tooltips!
;
;========================================================================================
; Recipe for Tasty Tooltips (serves millions)
;
; Since the information at MSDN isn't all in one place, and since some of it appears to be
; inaccurate (or at least incomplete), here's a recipe that'll work to give you tooltips:
;
; 0. Prepare the Common Controls DLL by calling InitCommonControlsEx(), specifying
;    ICC_BAR_CLASSES (at least; you can add other classes of controls as well.)
; 1. Create the toolbar window with the TBSTYLE_LIST and TBSTYLE_TOOLTIPS styles.
; 2. Create the toolbar as usual, loading bitmap(s) (if any), creating buttons, etc.
; 3. Set the TBSTYLE_EX_MIXEDBUTTONS "extended style" (by sending a SETEXTENDEDSTYLE
;    message to the control).
; 4. Handle the TTN_NEEDTEXT (alias TTN_GETDISPINFO) notification. This will come as a code
;    in a WM_NOTIFY message. Look at the control ID in the NMHDR header of the message;
;    if it matches your toolbar button, then all you need to do is to specify the text
;    for the tooltip. (This can be done either by copying the text into the NMTTDISPINFO
;    structure itself, or by giving it the address of a string in your application (or as
;    the handle to a string resource). You should also set the TTF_DI_SETITEM flag (in the
;    "uFlags" member) so that Windows will save the text and not bother you again.
;
; (Apparently) erroneous MSDN information:
;
; They tell us to put pointers to tooltip text in the "iString" member of the TBBUTTON
; structure, and to send the TB_SETMAXTEXTROWS message to set the max. # of rows to zero.
; Both of these things appear to be completely unnecessary (at least my tooltips work
; fine without them).
;
; It is also not necessary to respond to the TBN_GETINFOTIP notification (only
; TTN_NEEDTEXT/TTN_GETDISPINFO needs to be handled).
;
; That's it!
;========================================================================================


	include \masm32\include\masm32rt.inc


;============================================
; Defines, macros, prototypes, etc.
;============================================

WinMain			PROTO :DWORD

$mainWindowWidth	EQU 500
$mainWindowHeight	EQU 800

$toolbarWidth		EQU 192
$toolbarHeight		EQU 24

$numToolbarButtons	EQU 8

$toolbarBitmapID	EQU 500

;===== Window styles: =====
$mainWinAttrs		EQU WS_OVERLAPPEDWINDOW OR WS_CLIPCHILDREN
$toolbarAttrs		EQU WS_CHILD OR WS_VISIBLE OR WS_BORDER OR TBSTYLE_LIST OR TBSTYLE_TOOLTIPS

$BTNstates		EQU TBSTATE_ENABLED

$buttonWidth		EQU 24
$buttonHeight		EQU 24

;===== Toolbar button IDs: =====
$TBbutton1		EQU  1500
$TBbutton2		EQU  1501
$TBbutton3		EQU  1502
$TBbutton4		EQU  1503
$TBbutton5		EQU  1504
$TBbutton6		EQU  1505
$TBbutton7		EQU  1506
$TBbutton8		EQU  1507

;===== Window background colors: =====
$bkRED			EQU 254
$bkGRN			EQU 243
$bkBLUE			EQU 199

$BkColor		EQU $bkRED OR ($bkGRN SHL 8) OR ($bkBLUE SHL 16)

;============================================
; HERE BE DATA
;============================================
.data

InstanceHandle		HWND ?
MainWinHandle		HWND ?
ToolbarHandle		HWND ?

ToolbarBitmapHandle	HANDLE ?

ToolTipTextPtrs		DD BTN1text, BTN2text, BTN3text, BTN4text
			DD BTN5text, BTN6text, BTN7text, BTN8text

CCinitStruct	INITCOMMONCONTROLSEX <SIZEOF INITCOMMONCONTROLSEX, ICC_BAR_CLASSES>

;===== Toolbar button (TBBUTTON) structures:

ToolbarButtonStructs	LABEL DWORD
	TBBUTTON <0, $TBbutton1, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <1, $TBbutton2, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <2, $TBbutton3, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <3, $TBbutton4, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <4, $TBbutton5, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <5, $TBbutton6, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <6, $TBbutton7, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>
	TBBUTTON <7, $TBbutton8, $BTNstates, BTNS_BUTTON, 0, NULL, NULL>

ToolbarAddBitmapStruct	TBADDBITMAP <0, $toolbarBitmapID>

MainClassName		DB "skeleton", 0
ToolbarClassName	DB "ToolbarWindow32", 0

MainTitleText		DB "Toolbar Test Bed", 0

BTN1text		DB "whole note", 0
BTN2text		DB "half note", 0
BTN3text		DB "quarter note", 0
BTN4text		DB "8th note", 0
BTN5text		DB "whole rest", 0
BTN6text		DB "half rest", 0
BTN7text		DB "quarter rest", 0
BTN8text		DB "8th rest", 0

heremsg			db "Here I is!", 0


;============================================
; CODE LIVES HERE
;============================================
.code


start:
	INVOKE	GetModuleHandle, NULL
	MOV	InstanceHandle, EAX

	INVOKE	WinMain, EAX
	INVOKE	ExitProcess, EAX


;====================================================================
; Mainline proc
;====================================================================

WinMain	PROC hInst:DWORD
	LOCAL	wc:WNDCLASSEX, msg:MSG, brush:HBRUSH, wX:DWORD, wY:DWORD

;	INVOKE	LoadBitmap, hInst, $toolbarBitmapID
;	MOV	ToolbarBitmapHandle, EAX

	INVOKE	InitCommonControlsEx, OFFSET CCinitStruct

; Create  brush to set background color:
	INVOKE	CreateSolidBrush, $BkColor
	MOV	brush, EAX

; Register class for parent window:
	MOV	wc.cbSize, SIZEOF WNDCLASSEX
	MOV	wc.style,  CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
	MOV	wc.lpfnWndProc, OFFSET MainWindowProc
	MOV	wc.cbClsExtra, NULL
	MOV	wc.cbWndExtra, NULL
	MOV	EAX, hInst
	MOV	wc.hInstance, EAX
	MOV	EAX, brush
	MOV	wc.hbrBackground, EAX
	MOV	wc.lpszMenuName,  NULL
	MOV	wc.lpszClassName, OFFSET MainClassName
;	INVOKE	LoadIcon, hInst, 500    ; icon ID
;	MOV	wc.hIcon, EAX
	MOV	wc.hIcon, NULL
	INVOKE	LoadCursor, NULL, IDC_ARROW
	MOV	wc.hCursor, EAX
	MOV	wc.hIconSm, 0
	INVOKE	RegisterClassEx, ADDR wc

	INVOKE	GetSystemMetrics, SM_CXSCREEN
	MOV	EDX, $mainWindowWidth
	CALL	CenterDim
	MOV	wX, EAX
	INVOKE	GetSystemMetrics, SM_CYSCREEN
	MOV	EDX, $mainWindowHeight
	CALL	CenterDim
	MOV	wY, EAX

; Create main window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR MainClassName, ADDR MainTitleText,
		$mainWinAttrs, wX, wY, $mainWindowWidth, $mainWindowHeight,
		NULL, NULL, hInst, NULL
	MOV	MainWinHandle, EAX
	INVOKE	ShowWindow, EAX, SW_SHOWNORMAL

; Create toolbar window:
	INVOKE	CreateWindowEx, WS_EX_OVERLAPPEDWINDOW, ADDR ToolbarClassName, NULL,
		$toolbarAttrs, 0, 0, $toolbarWidth+10, $toolbarHeight+10,
		MainWinHandle, NULL, hInst, NULL
	MOV	ToolbarHandle, EAX

; Set up toolbar:
; Set size of TBUTTON structure:
	INVOKE	SendMessage, ToolbarHandle, TB_BUTTONSTRUCTSIZE, SIZEOF TBBUTTON, 0
; Set size of  bitmap:
	INVOKE	SendMessage, ToolbarHandle, TB_SETBITMAPSIZE, 0, $buttonWidth OR ($buttonHeight SHL 16)
; Add bitmap:
	MOV	EAX, InstanceHandle
	MOV	ToolbarAddBitmapStruct.hInst, EAX
	INVOKE	SendMessage, ToolbarHandle, TB_ADDBITMAP, $numToolbarButtons, OFFSET ToolbarAddBitmapStruct
; Add buttons:
	INVOKE	SendMessage, ToolbarHandle, TB_ADDBUTTONS, $numToolbarButtons, OFFSET ToolbarButtonStructs
; Set button size:
	INVOKE	SendMessage, ToolbarHandle, TB_SETBUTTONSIZE, 0, $buttonWidth OR ($buttonHeight SHL 16)

; Now set up toolbar tip stuff:
; MSDN says this needs done, but it seems to work fine without it:
;	INVOKE	SendMessage, ToolbarHandle, TB_SETMAXTEXTROWS, 0, 0	;Set text rows = 0.
	INVOKE	SendMessage, ToolbarHandle, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_MIXEDBUTTONS

;============= Message loop ===================
msgloop:
	INVOKE	GetMessage, ADDR msg, NULL, 0, 0
	OR	EAX, EAX				;EAX = 0 = exit
	JZ	exit
	INVOKE	TranslateMessage, ADDR msg
	INVOKE	DispatchMessage, ADDR msg
	JMP	msgloop

exit:	MOV	EAX, msg.wParam
	RET

WinMain		ENDP


;====================================================================
; Main Window Proc
;====================================================================

MainWindowProc	PROC hWin:DWORD, uMsg:DWORD, wParam:DWORD, lParam:DWORD

	MOV	EAX, uMsg
	CMP	EAX, WM_COMMAND
	JE	dowincmd
	CMP	EAX, WM_CREATE
	JE	docreate
	CMP	EAX, WM_NOTIFY
	JE	do_notify
	CMP	EAX, WM_CLOSE
	JE	doclose

dodefault:
; use DefWindowProc for all other messages:
	INVOKE	DefWindowProc, hWin, uMsg, wParam, lParam
	RET

dowincmd:

docreate:
	XOR	EAX, EAX
	RET

do_notify:

; If this is a notification from a toolbar button control requesting tooltip
; info, the data sent will be in a NMTTDISPINFO structure, with a NMHDR structure
; at its head.

	MOV	EDX, lParam			;--> NMTTDISPINFO struct.
; It is NOT necessary to respond to this notification, contrary to MSDN:
;	CMP	[EDX + NMHDR.code], TBN_GETINFOTIP
;	JE	@F
	CMP	[EDX + NMHDR.code], TTN_NEEDTEXT ;aka TTN_GETDISPINFO
	JNE	dodefault
	MOV	EAX, [EDX + NMHDR.idFrom]	;Could be the ID of one of our controls.
	CMP	EAX, $TBbutton1
	JB	dodefault
	CMP	EAX, $TBbutton8
	JA	dodefault

; At this point, we have a TTN_NEEDTEXT message for one of our buttons:
	SUB	EAX, $TBbutton1			;Get offset from 1st button ID:
	SHL	EAX, 2				;convert to DWORD offset
	MOV	EAX, [EAX + OFFSET ToolTipTextPtrs]
	MOV	[EDX + NMTTDISPINFO.lpszText], EAX	;Ptr. to tooltip text.
	MOV	[EDX + NMTTDISPINFO.lpszText], EAX	;Ptr. to tooltip text.
	MOV	[EDX + NMTTDISPINFO.uFlags], TTF_DI_SETITEM	;Don't bother me again!
	RET					;No particular return value needed.

;@@:	jmp	dodefault

doclose:
	INVOKE	PostQuitMessage, NULL
	MOV	EAX, TRUE
	RET


MainWindowProc	ENDP


;====================================================================
; CenterDim
;
; Returns half screen dimension (X or Y) minus half window dimension
;
; On entry,
;	EAX = screen dimension
;	EDX = window dimension
;
; Returns:
;	sdim/2 - wdim/2
;====================================================================

CenterDim	PROC

	SHR	EAX, 1				;divide screen dimension by 2
	SHR	EDX, 1				;divide window dimension by 2
	SUB	EAX, EDX
	RET					;Return w/# in EAX

CenterDim	ENDP


	END	start

